﻿using System;
using System.IO;
using System.Text;
using System.Xml;
using DevDefined.OAuth.Consumer;
using DevDefined.OAuth.Framework;
using System.Net;

namespace SamplePluginUsingOAuth
{
    /// <summary>
    /// Handles call to appservice rest methods
    /// </summary>
    public class AppServiceRestClient
    {
        private const string LicenseServiceMethodName = "AppLicensesForUser";
        private const string OrganisationServiceMethodName = "Person/{0}/Organizations";
        private const string PersonServiceMethodName = "Person";

        public string GetLicensesForUser(string token, string tokenSecret)
        {
            string response = CallRestApiWithOAuth(token, tokenSecret, string.Format("{0}/{1}", AppSettings.RestapiAppServiceUrl, LicenseServiceMethodName));
            return response;
        }

        public string GetOrganisationsForUser(string token, string tokenSecret, int personId)
        {
            string methodUrl = string.Format("{0}/{1}", AppSettings.RestapiPersonServiceUrl,
                                             string.Format(OrganisationServiceMethodName, personId));
            string response = CallRestApiWithOAuth(token, tokenSecret, methodUrl);
            return response;
        }

        public string GetPerson(string token, string tokenSecret)
        {
            string methodUrl = string.Format("{0}/{1}", AppSettings.RestapiPersonServiceUrl, PersonServiceMethodName);
            string response = CallRestApiWithOAuth(token, tokenSecret, methodUrl);
            return response;
        }

        private static string CallRestApiWithOAuth(string token, string tokenSecret, string methodUrl)
        {
            // Setup the context for our consumer with appkey and shared secret
            OAuthConsumerContext oAuthConsumerContext = new OAuthConsumerContext
                                                            {
                                                                ConsumerKey = AppSettings.ItslPluginApplicationKey,
                                                                ConsumerSecret = AppSettings.ItslPluginSharedSecret,
                                                                SignatureMethod = "HMAC-SHA1"
                                                            };
            // setup session - we don't need the requestTokenUrl, authorizeUrl and accessTokenUrl since we've already got the access token
            OAuthSession oAuthSession = new OAuthSession(oAuthConsumerContext, "http://notneeded", "http://notneeded", "http://notneeded");

            // Setup the token
            TokenBase oAuthtoken = new TokenBase
                                       {
                                           ConsumerKey = AppSettings.ItslPluginApplicationKey,
                                           Token = token,
                                           TokenSecret = tokenSecret
                                       };
            // Setup request
            IConsumerRequest request = oAuthSession.Request(oAuthtoken);
            request.Context.RequestMethod = "GET";
            request.Context.RawUri = new Uri(methodUrl);

            HttpWebRequest webRequest = request.ToWebRequest();
            // Send request and read response.
            string response = string.Empty;
            try
            {
                WebResponse resp = webRequest.GetResponse();
                using (Stream responseStream = resp.GetResponseStream())
                {
                    if (responseStream != null)
                    {
                        //TODO: Make sure that encoding is ok.
                        using (StreamReader responseStreamReader = new StreamReader(responseStream, Encoding.UTF8))
                        {
                            response = responseStreamReader.ReadToEnd();
                        }
                        response = FormatXml(response);
                    }
                }
            }
            catch(Exception e)
            {
                response = "Got exception when reading from request. Exception is: " + e;
            }
            return response;
        }

        /// <summary>
        /// Formats the XML.
        /// </summary>
        /// <param name="xml">The XML.</param>
        /// <returns></returns>
        private static string FormatXml(string xml)
        {
            if (string.IsNullOrEmpty(xml))
                return string.Empty;

            try
            {
                XmlDocument xmlDocument = new XmlDocument();
                xmlDocument.LoadXml(xml);

                StringBuilder stringBuilder = new StringBuilder();

                using (StringWriter stringWriter = new StringWriter(stringBuilder))
                {
                    using (XmlTextWriter xmlTextWriter = new XmlTextWriter(stringWriter))
                    {
                        xmlTextWriter.Formatting = Formatting.Indented;
                        xmlDocument.WriteTo(xmlTextWriter);
                    }
                }

                return stringBuilder.ToString();
            }
            catch (Exception)
            {
                return xml;
            }

        }
    }
}
