﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using Itslearning.Platform.RestApi.Sdk.Common;

namespace SamplePluginUsingOAuth
{
    public class SignatureValidator
    {
        /// <summary>
        /// Checks signature for new plugin sso
        /// </summary>
        /// <param name="stringToSign"></param>
        /// <param name="sharedSecret"></param>
        /// <param name="timeoutInSeconds"></param>
        /// <param name="signatureFromQueryString"></param>
        /// <param name="timeStampFromQueryString"></param>
        /// <returns></returns>
        public static bool IsPluginSignatureValidNewSso(string stringToSign, string sharedSecret, int timeoutInSeconds, string signatureFromQueryString, string timeStampFromQueryString)
        {
            string stringToSignWithSharedSecret = stringToSign + sharedSecret;
            string computedSignature = CryptographyHelper.ComputeHash(stringToSignWithSharedSecret);
            bool signatureVerified = string.Equals(signatureFromQueryString, computedSignature, StringComparison.InvariantCulture);
            if (!signatureVerified)
            {
                return false;
            }
            // Check timestamp
            DateTime timeStamp;
            if (!DateTime.TryParseExact(timeStampFromQueryString, "s", CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal | DateTimeStyles.AdjustToUniversal, out timeStamp))
            {
                return false;
            }

            TimeSpan span = DateTime.UtcNow - timeStamp;
            return Math.Abs((long)span.TotalSeconds) <= timeoutInSeconds;
        }


    }
}